@@ -22,6 +22,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/stickers/data_custom_emoji.h"
 #include "history/view/history_view_schedule_box.h"
 #include "lang/lang_keys.h"
+#include "main/main_app_config.h"
 #include "main/main_session.h"
 #include "menu/menu_send.h"
 #include "ui/controls/emoji_button.h"
@@ -113,7 +114,7 @@ private:
 		void setPlaceholder() const;
 		void removePlaceholder() const;
 
-		not_null<Ui::InputField*> field() const;
+		[[nodiscard]] not_null<Ui::InputField*> field() const;
 
 		[[nodiscard]] PollAnswer toPollAnswer(int index) const;
 
@@ -510,7 +511,8 @@ Options::Options(
 }
 
 bool Options::full() const {
-	return (_list.size() == kMaxOptionsCount);
+	const auto limit = _controller->session().appConfig().pollOptionsLimit();
+	return (_list.size() >= limit);
 }
 
 bool Options::hasOptions() const {
@@ -1028,8 +1030,10 @@ object_ptr<Ui::RpWidget> CreatePollBox::setupContent() {
 		setCloseByEscape(!count);
 		setCloseByOutsideClick(!count);
 	}) | rpl::map([=](int count) {
-		return (count < kMaxOptionsCount)
-			? tr::lng_polls_create_limit(tr::now, lt_count, kMaxOptionsCount - count)
+		const auto appConfig = &_controller->session().appConfig();
+		const auto max = appConfig->pollOptionsLimit();
+		return (count < max)
+			? tr::lng_polls_create_limit(tr::now, lt_count, max - count)
 			: tr::lng_polls_create_maximum(tr::now);
 	}) | rpl::after_next([=] {
 		container->resizeToWidth(container->widthNoMargins());
