@@ -902,7 +902,7 @@ void PreviewBox(
 
 	const auto outer = box->addRow(
 		ChatBackPreview(box, size.height(), back),
-		{});
+		style::margins());
 
 	struct Hiding {
 		not_null<Ui::RpWidget*> widget;
@@ -1079,26 +1079,21 @@ void PreviewBox(
 	auto text = state->selected.value(
 	) | rpl::map(SectionAbout) | rpl::flatten_latest();
 
-	const auto padding = st::premiumPreviewAboutPadding;
-	const auto available = size.width() - padding.left() - padding.right();
-	auto titleLabel = object_ptr<Ui::FlatLabel>(
-		box,
-		std::move(title),
-		st::premiumPreviewAboutTitle);
-	titleLabel->resizeToWidth(available);
 	box->addRow(
-		object_ptr<Ui::CenterWrap<Ui::FlatLabel>>(
+		object_ptr<Ui::FlatLabel>(
 			box,
-			std::move(titleLabel)),
-		st::premiumPreviewAboutTitlePadding);
-	auto textLabel = object_ptr<Ui::FlatLabel>(
-		box,
-		std::move(text),
-		st::premiumPreviewAbout);
-	textLabel->resizeToWidth(available);
+			std::move(title),
+			st::premiumPreviewAboutTitle),
+		st::premiumPreviewAboutTitlePadding,
+		style::al_top);
 	box->addRow(
-		object_ptr<Ui::CenterWrap<Ui::FlatLabel>>(box, std::move(textLabel)),
-		padding);
+		object_ptr<Ui::FlatLabel>(
+			box,
+			std::move(text),
+			st::premiumPreviewAbout),
+		st::premiumPreviewAboutPadding,
+		style::al_top
+	)->setTryMakeSimilarLines(true);
 	box->addRow(
 		CreateSwitch(box->verticalLayout(), &state->selected, state->order),
 		st::premiumDotsMargin);
