@@ -116,13 +116,12 @@ void ConferenceCallJoinConfirm(
 		st::boxRowPadding + st::confcallLinkHeaderIconPadding);
 
 	box->addRow(
-		object_ptr<Ui::CenterWrap<Ui::FlatLabel>>(
+		object_ptr<Ui::FlatLabel>(
 			box,
-			object_ptr<Ui::FlatLabel>(
-				box,
-				tr::lng_confcall_join_title(),
-				st::boxTitle)),
-		st::boxRowPadding + st::confcallLinkTitlePadding);
+			tr::lng_confcall_join_title(),
+			st::boxTitle),
+		st::boxRowPadding + st::confcallLinkTitlePadding,
+		style::al_top);
 	const auto wrapName = [&](not_null<PeerData*> peer) {
 		return rpl::single(Ui::Text::Bold(peer->shortName()));
 	};
@@ -136,7 +135,8 @@ void ConferenceCallJoinConfirm(
 					Ui::Text::RichLangValue)
 				: tr::lng_confcall_join_text(Ui::Text::RichLangValue)),
 			st::confcallLinkCenteredText),
-		st::boxRowPadding
+		st::boxRowPadding,
+		style::al_top
 	)->setTryMakeSimilarLines(true);
 
 	const auto &participants = call->participants();
@@ -209,7 +209,8 @@ void ConferenceCallJoinConfirm(
 				box,
 				std::move(text),
 				st::confcallLinkCenteredText),
-			st::boxRowPadding
+			st::boxRowPadding,
+			style::al_top
 		)->setTryMakeSimilarLines(true);
 	}
 	const auto joinAndClose = [=] {
@@ -280,7 +281,8 @@ void ShowConferenceCallLinkBox(
 					MTPphone_ToggleGroupCallSettings(
 						MTP_flags(Flag::f_reset_invite_hash),
 						call->input(),
-						MTPbool()) // join_muted
+						MTPBool(), // join_muted
+						MTPBool()) // messages_enabled
 				).done([=](const MTPUpdates &result) {
 					call->session().api().applyUpdates(result);
 					ShowConferenceCallLinkBox(show, call, args);
@@ -321,13 +323,12 @@ void ShowConferenceCallLinkBox(
 			Info::BotStarRef::CreateLinkHeaderIcon(box, &call->session()),
 			st::boxRowPadding + st::confcallLinkHeaderIconPadding);
 		box->addRow(
-			object_ptr<Ui::CenterWrap<Ui::FlatLabel>>(
+			object_ptr<Ui::FlatLabel>(
 				box,
-				object_ptr<Ui::FlatLabel>(
-					box,
-					tr::lng_confcall_link_title(),
-					st.box ? st.box->title : st::boxTitle)),
-			st::boxRowPadding + st::confcallLinkTitlePadding);
+				tr::lng_confcall_link_title(),
+				st.box ? st.box->title : st::boxTitle),
+			st::boxRowPadding + st::confcallLinkTitlePadding,
+			style::al_top);
 		box->addRow(
 			object_ptr<Ui::FlatLabel>(
 				box,
@@ -335,7 +336,8 @@ void ShowConferenceCallLinkBox(
 				(st.centerLabel
 					? *st.centerLabel
 					: st::confcallLinkCenteredText)),
-			st::boxRowPadding
+			st::boxRowPadding,
+			style::al_top
 		)->setTryMakeSimilarLines(true);
 
 		Ui::AddSkip(box->verticalLayout(), st::defaultVerticalListSkip * 2);
