@@ -1182,12 +1182,6 @@ void ChannelData::setStoriesState(StoriesState state) {
 	}
 }
 
-void ChannelData::processTopics(const MTPVector<MTPForumTopic> &topics) {
-	if (const auto forum = this->forum()) {
-		forum->applyReceivedTopics(topics);
-	}
-}
-
 int ChannelData::levelHint() const {
 	return _levelHint;
 }
