@@ -246,6 +246,30 @@ UserData *UserFromInputMTP(
 	});
 }
 
+Ui::ColorCollectible ParseColorCollectible(
+		const MTPDpeerColorCollectible &data) {
+	return {
+		.collectibleId = data.vcollectible_id().v,
+		.giftEmojiId = data.vgift_emoji_id().v,
+		.backgroundEmojiId = data.vbackground_emoji_id().v,
+		.accentColor = Ui::ColorFromSerialized(data.vaccent_color()),
+		.strip = ranges::views::all(
+			data.vcolors().v
+		) | ranges::views::transform(
+			&Ui::ColorFromSerialized
+		) | ranges::to_vector,
+		.darkAccentColor = Ui::MaybeColorFromSerialized(
+			data.vdark_accent_color()).value_or(QColor(0, 0, 0, 0)),
+		.darkStrip = (data.vdark_colors()
+			? ranges::views::all(
+				data.vdark_colors()->v
+			) | ranges::views::transform(
+				&Ui::ColorFromSerialized
+			) | ranges::to_vector
+			: std::vector<QColor>()),
+	};
+}
+
 } // namespace Data
 
 PeerClickHandler::PeerClickHandler(not_null<PeerData*> peer)
@@ -693,7 +717,9 @@ bool PeerData::canCreateTodoLists() const {
 }
 
 bool PeerData::canCreateTopics() const {
-	if (const auto channel = asChannel()) {
+	if (const auto bot = asBot()) {
+		return bot->isForum();
+	} else if (const auto channel = asChannel()) {
 		return channel->isForum()
 			&& !channel->amRestricted(ChatRestriction::CreateTopics);
 	}
@@ -701,7 +727,9 @@ bool PeerData::canCreateTopics() const {
 }
 
 bool PeerData::canManageTopics() const {
-	if (const auto channel = asChannel()) {
+	if (const auto bot = asBot()) {
+		return bot->isForum();
+	} else if (const auto channel = asChannel()) {
 		return channel->isForum()
 			&& (channel->amCreator()
 				|| (channel->adminRights() & ChatAdminRight::ManageTopics));
@@ -915,6 +943,18 @@ void PeerData::setBarSettings(const MTPPeerSettings &data) {
 	});
 }
 
+void PeerData::setBarSettings(PeerBarSettings which) {
+	const auto was = hideLinks();
+	_barSettings.set(which);
+	if (was && !hideLinks()) {
+		if (const auto history = owner().historyLoaded(this)) {
+			crl::on_main(&history->session(), [=] {
+				history->refreshHiddenLinksItems();
+			});
+		}
+	}
+}
+
 int PeerData::paysPerMessage() const {
 	return _barDetails ? _barDetails->paysPerMessage : 0;
 }
@@ -936,6 +976,14 @@ void PeerData::clearPaysPerMessage() {
 	}
 }
 
+bool PeerData::hideLinks() const {
+	//if (!isUser()) {
+	//	return false;
+	//}
+	const auto settings = barSettings();
+	return !settings || (*settings & PeerBarSetting::ReportSpam);
+}
+
 QString PeerData::requestChatTitle() const {
 	return _barDetails ? _barDetails->requestChatTitle : QString();
 }
@@ -974,13 +1022,6 @@ TimeId PeerData::photoChangeDate() const {
 	return _barDetails ? _barDetails->photoChangeDate : 0;
 }
 
-bool PeerData::changeColorIndex(
-		const tl::conditional<MTPint> &cloudColorIndex) {
-	return cloudColorIndex
-		? changeColorIndex(cloudColorIndex->v)
-		: clearColorIndex();
-}
-
 bool PeerData::changeBackgroundEmojiId(
 		const tl::conditional<MTPlong> &cloudBackgroundEmoji) {
 	return changeBackgroundEmojiId(cloudBackgroundEmoji
@@ -988,15 +1029,29 @@ bool PeerData::changeBackgroundEmojiId(
 		: DocumentId());
 }
 
+bool PeerData::changeColorCollectible(
+		const tl::conditional<MTPPeerColor> &cloudColor) {
+	if (!cloudColor) {
+		return clearColorCollectible();
+	}
+	return cloudColor->match([&](const MTPDpeerColorCollectible &data) {
+		return changeColorCollectible(Data::ParseColorCollectible(data));
+	}, [&](const MTPDpeerColor &) {
+		return clearColorCollectible();
+	}, [&](const MTPDinputPeerColorCollectible &) {
+		return clearColorCollectible();
+	});
+}
+
 bool PeerData::changeColor(
 		const tl::conditional<MTPPeerColor> &cloudColor) {
 	const auto changed1 = cloudColor
-		? changeColorIndex(cloudColor->data().vcolor())
+		? changeColorIndex(Data::ColorIndexFromColor(cloudColor))
 		: clearColorIndex();
-	const auto changed2 = changeBackgroundEmojiId(cloudColor
-		? cloudColor->data().vbackground_emoji_id().value_or_empty()
-		: DocumentId());
-	return changed1 || changed2;
+	const auto changed2 = changeBackgroundEmojiId(
+		Data::BackgroundEmojiIdFromColor(cloudColor));
+	const auto changed3 = changeColorCollectible(cloudColor);
+	return changed1 || changed2 || changed3;
 }
 
 void PeerData::fillNames() {
@@ -1305,6 +1360,25 @@ bool PeerData::isUsernameEditable(QString username) const {
 	return false;
 }
 
+bool PeerData::changeColorCollectible(Ui::ColorCollectible data) {
+	if (!_colorCollectible || (*_colorCollectible != data)) {
+		// We don't reuse allocated object because in ChatStyle we
+		// cache colors using std::weak_ptr as a key.
+		_colorCollectible = std::make_shared<Ui::ColorCollectible>(
+			std::move(data));
+		return true;
+	}
+	return false;
+}
+
+bool PeerData::clearColorCollectible() {
+	if (!_colorCollectible) {
+		return false;
+	}
+	_colorCollectible = nullptr;
+	return true;
+}
+
 bool PeerData::changeColorIndex(uint8 index) {
 	index %= Ui::kColorIndexCount;
 	if (_colorIndexCloud && _colorIndex == index) {
@@ -1416,7 +1490,9 @@ bool PeerData::isBroadcast() const {
 }
 
 bool PeerData::isForum() const {
-	if (const auto channel = asChannel()) {
+	if (const auto bot = asBot()) {
+		return bot->isForum();
+	} else if (const auto channel = asChannel()) {
 		return channel->isForum();
 	}
 	return false;
@@ -1497,7 +1573,9 @@ Ui::BotVerifyDetails *PeerData::botVerifyDetails() const {
 }
 
 Data::Forum *PeerData::forum() const {
-	if (const auto channel = asChannel()) {
+	if (const auto bot = asBot()) {
+		return bot->forum();
+	} else if (const auto channel = asChannel()) {
 		return channel->forum();
 	}
 	return nullptr;
@@ -1529,6 +1607,28 @@ Data::SavedSublist *PeerData::monoforumSublistFor(
 	return nullptr;
 }
 
+bool PeerData::useSubsectionTabs() const {
+	if (const auto bot = asBot()) {
+		return bot->isForum();
+	} else if (const auto channel = asChannel()) {
+		return channel->useSubsectionTabs();
+	}
+	return false;
+}
+
+bool PeerData::viewForumAsMessages() const {
+	if (const auto channel = asChannel()) {
+		return channel->viewForumAsMessages();
+	}
+	return false;
+}
+
+void PeerData::processTopics(const MTPVector<MTPForumTopic> &topics) {
+	if (const auto forum = this->forum()) {
+		forum->applyReceivedTopics(topics);
+	}
+}
+
 bool PeerData::allowsForwarding() const {
 	if (isUser()) {
 		return true;
@@ -1736,24 +1836,23 @@ PeerId PeerData::groupCallDefaultJoinAs() const {
 	return 0;
 }
 
-void PeerData::setThemeEmoji(const QString &emoticon) {
-	if (_themeEmoticon == emoticon) {
+void PeerData::setThemeToken(const QString &token) {
+	if (_themeToken == token) {
+		return;
+	} else if (!token.startsWith(u"gift:"_q)
+		&& Ui::Emoji::Find(_themeToken) == Ui::Emoji::Find(token)) {
+		_themeToken = token;
 		return;
 	}
-	if (Ui::Emoji::Find(_themeEmoticon) == Ui::Emoji::Find(emoticon)) {
-		_themeEmoticon = emoticon;
-		return;
+	_themeToken = token;
+	if (!token.isEmpty() && !owner().cloudThemes().themeForToken(token)) {
+		owner().cloudThemes().refreshChatThemesFor(token);
 	}
-	_themeEmoticon = emoticon;
-	if (!emoticon.isEmpty()
-		&& !owner().cloudThemes().themeForEmoji(emoticon)) {
-		owner().cloudThemes().refreshChatThemes();
-	}
-	session().changes().peerUpdated(this, UpdateFlag::ChatThemeEmoji);
+	session().changes().peerUpdated(this, UpdateFlag::ChatThemeToken);
 }
 
-const QString &PeerData::themeEmoji() const {
-	return _themeEmoticon;
+const QString &PeerData::themeToken() const {
+	return _themeToken;
 }
 
 void PeerData::setWallPaper(
@@ -1968,4 +2067,30 @@ FullMsgId ResolveMinPinnedId(
 	}
 }
 
+uint64 BackgroundEmojiIdFromColor(const MTPPeerColor *color) {
+	if (!color) {
+		return 0;
+	}
+	return color->match([](const MTPDpeerColor &data) -> uint64 {
+		return data.vbackground_emoji_id().value_or_empty();
+	}, [](const MTPDpeerColorCollectible &data) -> uint64 {
+		return data.vbackground_emoji_id().v;
+	}, [](const MTPDinputPeerColorCollectible &data) -> uint64 {
+		return 0;
+	});
+}
+
+uint8 ColorIndexFromColor(const MTPPeerColor *color) {
+	if (!color) {
+		return 0;
+	}
+	return color->match([](const MTPDpeerColor &data) -> uint8 {
+		return data.vcolor().value_or_empty();
+	}, [](const MTPDpeerColorCollectible &data) -> uint8 {
+		return 0;
+	}, [](const MTPDinputPeerColorCollectible &) -> uint8 {
+		return 0;
+	});
+}
+
 } // namespace Data
