@@ -121,6 +121,11 @@ struct SentFromScheduled {
 	MsgId sentId = 0;
 };
 
+struct RecentSelfForwards {
+	PeerId fromPeerId = 0;
+	MessageIdsList ids;
+};
+
 class Session final {
 public:
 	using ViewElement = HistoryView::Element;
@@ -580,7 +585,9 @@ public:
 		NewMessageType type);
 
 	[[nodiscard]] int unreadBadge() const;
+	[[nodiscard]] int unreadWithMentionsBadge() const;
 	[[nodiscard]] bool unreadBadgeMuted() const;
+	[[nodiscard]] bool unreadWithMentionsBadgeMuted() const;
 	[[nodiscard]] int unreadBadgeIgnoreOne(Dialogs::Key key) const;
 	[[nodiscard]] bool unreadBadgeMutedIgnoreOne(Dialogs::Key key) const;
 	[[nodiscard]] int unreadOnlyMutedBadge() const;
@@ -885,6 +892,9 @@ public:
 	void setPendingStarsRating(StarsRatingPending value);
 	[[nodiscard]] StarsRatingPending pendingStarsRating() const;
 
+	void addRecentSelfForwards(const RecentSelfForwards &data);
+	[[nodiscard]] rpl::producer<RecentSelfForwards> recentSelfForwards() const;
+
 	void clearLocalStorage();
 
 private:
@@ -1253,6 +1263,8 @@ private:
 		not_null<PeerData*>,
 		NextToUpgradeGift> _nextForUpgradeGifts;
 
+	rpl::event_stream<RecentSelfForwards> _recentSelfForwards;
+
 	rpl::lifetime _lifetime;
 
 };
