@@ -18,6 +18,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "base/flags.h"
 
 namespace Data {
+class Forum;
 struct BotCommand;
 struct BusinessDetails;
 } // namespace Data
@@ -55,6 +56,11 @@ struct BotVerifierSettings {
 
 struct BotInfo {
 	BotInfo();
+	~BotInfo();
+
+	void ensureForum(not_null<UserData*> that);
+	[[nodiscard]] Data::Forum *forum() const;
+	[[nodiscard]] std::unique_ptr<Data::Forum> takeForumData();
 
 	QString description;
 	QString inlinePlaceholder;
@@ -92,6 +98,10 @@ struct BotInfo {
 	bool canManageEmojiStatus : 1 = false;
 	bool supportsBusiness : 1 = false;
 	bool hasMainApp : 1 = false;
+
+private:
+	std::unique_ptr<Data::Forum> _forum;
+
 };
 
 enum class UserDataFlag : uint32 {
@@ -122,6 +132,7 @@ enum class UserDataFlag : uint32 {
 	MessageMoneyRestrictionsKnown = (1 << 24),
 	ReadDatesPrivate = (1 << 25),
 	StoriesCorrespondent = (1 << 26),
+	Forum = (1ULL << 27),
 };
 inline constexpr bool is_flag_type(UserDataFlag) { return true; };
 using UserDataFlags = base::flags<UserDataFlag>;
@@ -188,6 +199,12 @@ public:
 	[[nodiscard]] bool messageMoneyRestrictionsKnown() const;
 	[[nodiscard]] bool canSendIgnoreMoneyRestrictions() const;
 	[[nodiscard]] bool readDatesPrivate() const;
+	[[nodiscard]] bool isForum() const {
+		return flags() & Flag::Forum;
+	}
+	[[nodiscard]] Data::Forum *forum() const {
+		return botInfo ? botInfo->forum() : nullptr;
+	}
 
 	void setStoriesCorrespondent(bool is);
 	[[nodiscard]] bool storiesCorrespondent() const;
@@ -280,6 +297,9 @@ public:
 	}
 	void setDisallowedGiftTypes(Api::DisallowedGiftTypes types);
 
+	[[nodiscard]] const TextWithEntities &note() const;
+	void setNote(const TextWithEntities &note);
+
 private:
 	auto unavailableReasons() const
 		-> const std::vector<Data::UnavailableReason> & override;
@@ -313,6 +333,7 @@ private:
 		= 0xFFFFFFFFFFFFFFFFULL;
 
 	Api::DisallowedGiftTypes _disallowedGiftTypes;
+	TextWithEntities _note;
 
 };
 
