@@ -41,7 +41,6 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "apiwrap.h"
 #include "styles/style_chat.h"
 #include "styles/style_chat_helpers.h"
-#include "styles/style_credits.h" // giftBoxByStarsStyle
 
 namespace Data {
 namespace {
@@ -101,10 +100,6 @@ private:
 		: FrameSizeFromTag(tag);
 }
 
-[[nodiscard]] QString InternalPrefix() {
-	return u"internal:"_q;
-}
-
 [[nodiscard]] QString UserpicEmojiPrefix() {
 	return u"userpic:"_q;
 }
@@ -570,8 +565,6 @@ std::unique_ptr<Ui::Text::CustomEmoji> CustomEmojiManager::create(
 		const auto original = data.mid(ForceStaticPrefix().size());
 		return std::make_unique<Ui::Text::FirstFrameEmoji>(
 			create(original, std::move(update), tag, sizeOverride));
-	} else if (data.startsWith(InternalPrefix())) {
-		return internal(data);
 	} else if (data.startsWith(UserpicEmojiPrefix())) {
 		const auto ratio = style::DevicePixelRatio();
 		const auto size = EmojiSizeFromTag(tag) / ratio;
@@ -619,26 +612,6 @@ std::unique_ptr<Ui::Text::CustomEmoji> CustomEmojiManager::create(
 	});
 }
 
-std::unique_ptr<Ui::Text::CustomEmoji> CustomEmojiManager::internal(
-		QStringView data) {
-	const auto v = data.mid(InternalPrefix().size()).split(',');
-	if (v.size() != 5 && v.size() != 1) {
-		return nullptr;
-	}
-	const auto index = v[0].toInt();
-	Assert(index >= 0 && index < _internalEmoji.size());
-
-	auto &info = _internalEmoji[index];
-	const auto padding = (v.size() == 5)
-		? QMargins(v[1].toInt(), v[2].toInt(), v[3].toInt(), v[4].toInt())
-		: QMargins();
-	return std::make_unique<Ui::CustomEmoji::Internal>(
-		data.toString(),
-		info.image,
-		padding,
-		info.textColor);
-}
-
 std::unique_ptr<Ui::Text::CustomEmoji> CustomEmojiManager::userpic(
 		QStringView data,
 		Fn<void()> update,
@@ -1021,65 +994,6 @@ uint64 CustomEmojiManager::coloredSetId() const {
 	return _coloredSetId;
 }
 
-TextWithEntities CustomEmojiManager::creditsEmoji(QMargins padding) {
-	return Ui::Text::SingleCustomEmoji(registerInternalEmoji(
-		u"builtin:credits_emoji"_q,
-		Ui::GenerateStars(st::normalFont->height, 1),
-		padding,
-		false));
-}
-
-TextWithEntities CustomEmojiManager::ministarEmoji(QMargins padding) {
-	return Ui::Text::SingleCustomEmoji(registerInternalEmoji(
-		u"builtin:ministar_emoji"_q,
-		Ui::GenerateStars(st::giftBoxByStarsStyle.font->height, 1),
-		padding,
-		false));
-}
-
-QString CustomEmojiManager::registerInternalEmoji(
-		const QString &key,
-		QImage emoji,
-		QMargins padding,
-		bool textColor) {
-	auto i = _imageEmoji.find(key);
-	if (i == end(_imageEmoji)) {
-		i = _imageEmoji.emplace(
-			key,
-			registerImageEmoji(std::move(emoji), textColor)).first;
-	}
-	return i->second + InternalPadding(padding);
-}
-
-QString CustomEmojiManager::registerImageEmoji(
-		QImage emoji,
-		bool textColor) {
-	_internalEmoji.push_back({ std::move(emoji), textColor });
-	return InternalPrefix() + QString::number(_internalEmoji.size() - 1);
-}
-
-QString CustomEmojiManager::registerInternalEmoji(
-		const style::icon &icon,
-		QMargins padding,
-		bool textColor) {
-	const auto i = _iconEmoji.find(&icon);
-	if (i != end(_iconEmoji)) {
-		return i->second + InternalPadding(padding);
-	}
-	auto image = QImage(
-		icon.size() * style::DevicePixelRatio(),
-		QImage::Format_ARGB32_Premultiplied);
-	image.fill(Qt::transparent);
-	image.setDevicePixelRatio(style::DevicePixelRatio());
-	auto p = QPainter(&image);
-	icon.paint(p, 0, 0, icon.width());
-	p.end();
-
-	const auto result = registerImageEmoji(std::move(image), textColor);
-	_iconEmoji.emplace(&icon, result);
-	return result + InternalPadding(padding);
-}
-
 [[nodiscard]] QString CustomEmojiManager::peerUserpicEmojiData(
 		not_null<PeerData*> peer,
 		QMargins padding,
