@@ -317,31 +317,32 @@ Row::~Row() {
 	clearTopicJumpRipple();
 }
 
-void Row::recountHeight(float64 narrowRatio, FilterId filterId) {
-	if (const auto history = _id.history()) {
-		const auto hasTags = _id.entry()->hasChatsFilterTags(filterId);
+const style::DialogRow &Row::ComputeSt(
+		not_null<const Entry*> entry,
+		FilterId filterId) {
+	if (const auto history = entry->asHistory()) {
+		const auto hasTags = entry->hasChatsFilterTags(filterId);
 		const auto wideRow = history->isForum()
 			|| history->amMonoforumAdmin();
-		_height = wideRow
-			? anim::interpolate(
-				hasTags
-					? st::taggedForumDialogRow.height
-					: st::forumDialogRow.height,
-				st::defaultDialogRow.height,
-				narrowRatio)
+		return wideRow
+			? (hasTags ? st::taggedForumDialogRow : st::forumDialogRow)
 			: hasTags
-			? anim::interpolate(
-				st::taggedDialogRow.height,
-				st::defaultDialogRow.height,
-				narrowRatio)
-			: st::defaultDialogRow.height;
-	} else if (_id.folder()) {
-		_height = st::defaultDialogRow.height;
-	} else if (_id.topic()) {
-		_height = st::forumTopicRow.height;
-	} else {
-		_height = st::defaultDialogRow.height;
+			? st::taggedDialogRow
+			: st::defaultDialogRow;
+	} else if (entry->asTopic()) {
+		return st::forumTopicRow;
 	}
+	return st::defaultDialogRow;
+}
+
+void Row::recountHeight(float64 narrowRatio, FilterId filterId) {
+	const auto &st = ComputeSt(_id.entry(), filterId);
+	_height = ((&st == &st::defaultDialogRow) || !_id.history())
+		? st::defaultDialogRow.height
+		: anim::interpolate(
+			st.height,
+			st::defaultDialogRow.height,
+			narrowRatio);
 }
 
 uint64 Row::sortKey(FilterId filterId) const {
