@@ -11,6 +11,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "apiwrap.h"
 #include "base/unixtime.h"
 #include "base/qt/qt_key_modifiers.h"
+#include "boxes/choose_filter_box.h"
 #include "boxes/peer_list_box.h"
 #include "core/application.h"
 #include "core/ui_integration.h"
@@ -20,6 +21,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/data_changes.h"
 #include "data/data_channel.h"
 #include "data/data_chat.h"
+#include "data/data_chat_filters.h"
 #include "data/data_download_manager.h"
 #include "data/data_folder.h"
 #include "data/data_peer_values.h"
@@ -68,6 +70,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "window/window_separate_id.h"
 #include "window/window_session_controller.h"
 #include "window/window_peer_menu.h"
+#include "styles/style_boxes.h"
 #include "styles/style_chat.h"
 #include "styles/style_dialogs.h"
 #include "styles/style_layers.h"
@@ -179,6 +182,20 @@ void FillEntryMenu(
 		controller->showPeerHistory(peer);
 	}, channel ? &st::menuIconChannel : &st::menuIconChatBubble);
 
+	const auto history = peer->owner().historyLoaded(peer);
+	if (history
+		&& history->owner().chatsFilters().has()
+		&& history->inChatList()) {
+		add(Ui::Menu::MenuCallback::Args{
+			.text = tr::lng_filters_menu_add(tr::now),
+			.handler = nullptr,
+			.icon = &st::menuIconAddToFolder,
+			.fillSubmenu = [&](not_null<Ui::PopupMenu*> menu) {
+				FillChooseFilterMenu(controller, menu, history);
+			},
+			.submenuSt = &st::foldersMenu,
+		});
+	}
 	const auto viewProfileText = group
 		? tr::lng_context_view_group(tr::now)
 		: channel
@@ -1427,20 +1444,13 @@ void Suggestions::setupTabs() {
 		},
 	};
 
-	auto helper = Ui::Text::CustomEmojiHelper();
 	auto sections = std::vector<TextWithEntities>();
 	for (const auto key : _tabKeys) {
 		const auto i = labels.find(key);
 		Assert(i != end(labels));
-		auto text = TextWithEntities{ i->second };
-		if (key.tab == Tab::Posts) {
-			text.append(' ').append(helper.paletteDependent(
-				Ui::Text::CustomEmojiTextBadge(
-					tr::lng_premium_summary_new_badge(tr::now))));
-		}
-		sections.push_back(std::move(text));
+		sections.push_back({ i->second });
 	}
-	_tabs->setSections(sections, helper.context());
+	_tabs->setSections(sections);
 	_tabs->sectionActivated(
 	) | rpl::start_with_next([=](int section) {
 		Assert(section >= 0 && section < _tabKeys.size());
