@@ -827,10 +827,24 @@ uint8 Element::colorIndex() const {
 	return data()->colorIndex();
 }
 
+auto Element::colorCollectible() const
+-> const std::shared_ptr<Ui::ColorCollectible> & {
+	return data()->colorCollectible();
+}
+
 uint8 Element::contentColorIndex() const {
 	return data()->contentColorIndex();
 }
 
+DocumentId Element::contentBackgroundEmojiId() const {
+	return data()->contentBackgroundEmojiId();
+}
+
+auto Element::contentColorCollectible() const
+-> const std::shared_ptr<Ui::ColorCollectible> & {
+	return data()->contentColorCollectible();
+}
+
 QDateTime Element::dateTime() const {
 	return _dateTime;
 }
@@ -965,6 +979,17 @@ bool Element::isLastAndSelfMessage() const {
 	return false;
 }
 
+void Element::addVerticalMargins(int top, int bottom) {
+	if (top || bottom) {
+		AddComponents(ViewAddedMargins::Bit());
+		const auto margins = Get<ViewAddedMargins>();
+		margins->top = top;
+		margins->bottom = bottom;
+	} else {
+		RemoveComponents(ViewAddedMargins::Bit());
+	}
+}
+
 void Element::setPendingResize() {
 	_flags |= Flag::NeedsResize;
 	if (_context == Context::History) {
@@ -1206,9 +1231,10 @@ auto Element::contextDependentServiceText() -> TextWithLinks {
 		return Ui::Text::Link(from->name(), index);
 	};
 	const auto placeholderLink = [&] {
-		return Ui::Text::Link(
-			tr::lng_action_topic_placeholder(tr::now),
-			topicUrl);
+		const auto linkText = history()->peer->isBot()
+			? tr::lng_action_topic_bot_thread(tr::now)
+			: tr::lng_action_topic_placeholder(tr::now);
+		return Ui::Text::Link(linkText, topicUrl);
 	};
 	const auto wrapTopic = [&](
 			const QString &title,
@@ -1323,13 +1349,16 @@ void Element::validateText() {
 	const auto item = data();
 	const auto media = item->media();
 	const auto storyMention = media && media->storyMention();
-	if (media && media->storyExpired()) {
+	const auto storyExpired = media && media->storyExpired();
+	const auto storyUnsupported = media && media->storyUnsupported();
+	if (storyExpired || storyUnsupported) {
 		_media = nullptr;
 		_textItem = item;
 		if (!storyMention) {
 			if (_text.isEmpty()) {
-				setTextWithLinks(Ui::Text::Italic(
-					tr::lng_forwarded_story_expired(tr::now)));
+				setTextWithLinks(Ui::Text::Italic(storyUnsupported
+					? tr::lng_stories_unsupported(tr::now)
+					: tr::lng_forwarded_story_expired(tr::now)));
 			}
 			return;
 		}
@@ -1614,8 +1643,8 @@ void Element::recountThreadBarInBlocks() {
 	const auto item = data();
 	const auto topic = item->topic();
 	const auto sublist = item->savedSublist();
-	const auto parentChat = (topic && topic->channel()->useSubsectionTabs())
-		? topic->channel().get()
+	const auto parentChat = (topic && topic->peer()->useSubsectionTabs())
+		? topic->peer().get()
 		: sublist
 		? sublist->parentChat()
 		: nullptr;
@@ -1629,7 +1658,7 @@ void Element::recountThreadBarInBlocks() {
 		if (const auto previous = previousDisplayedInBlocks()) {
 			const auto prev = previous->data();
 			if (const auto prevTopic = prev->topic()) {
-				Assert(prevTopic->channel() == parentChat);
+				Assert(prevTopic->peer() == parentChat);
 				const auto topicRootId = topic->rootId();
 				if (prevTopic->rootId() == topicRootId) {
 					return nullptr;
@@ -2360,9 +2389,6 @@ auto Element::takeReactionAnimations()
 	return {};
 }
 
-void Element::animateEffect(Ui::ReactionFlyAnimationArgs &&args) {
-}
-
 void Element::animateUnreadEffect() {
 }
 
