@@ -14,7 +14,6 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "history/view/history_view_element.h"
 #include "history/view/history_view_cursor_state.h"
 #include "ui/chat/chat_style.h"
-#include "ui/widgets/tooltip.h"
 #include "ui/dynamic_image.h"
 #include "ui/dynamic_thumbnails.h"
 #include "ui/painter.h"
@@ -28,19 +27,6 @@ namespace {
 
 constexpr auto kAdditionalPrizesWithLineOpacity = 0.6;
 
-[[nodiscard]] QSize CountOptimalTextSize(
-		const Ui::Text::String &text,
-		int minWidth,
-		int maxWidth) {
-	if (text.maxWidth() <= maxWidth) {
-		return { text.maxWidth(), text.minHeight() };
-	}
-	const auto height = text.countHeight(maxWidth);
-	return { Ui::FindNiceTooltipWidth(minWidth, maxWidth, [&](int width) {
-		return text.countHeight(width);
-	}), height };
-}
-
 } // namespace
 
 TextState MediaGenericPart::textState(
@@ -77,6 +63,7 @@ MediaGeneric::MediaGeneric(
 	MediaGenericDescriptor &&descriptor)
 : Media(parent)
 , _paintBg(std::move(descriptor.paintBg))
+, _fullAreaLink(descriptor.fullAreaLink)
 , _maxWidthCap(descriptor.maxWidth)
 , _service(descriptor.service)
 , _hideServiceText(descriptor.hideServiceText) {
@@ -85,10 +72,6 @@ MediaGeneric::MediaGeneric(
 			.object = std::move(part),
 		});
 	});
-	if (descriptor.serviceLink) {
-		parent->data()->setCustomServiceLink(
-			std::move(descriptor.serviceLink));
-	}
 }
 
 MediaGeneric::~MediaGeneric() {
@@ -116,10 +99,11 @@ QSize MediaGeneric::countCurrentSize(int newWidth) {
 	if (newWidth > maxWidth()) {
 		newWidth = maxWidth();
 	}
+	auto top = 0;
 	for (auto &entry : _entries) {
-		entry.object->resizeGetHeight(newWidth);
+		top += entry.object->resizeGetHeight(newWidth);
 	}
-	return { newWidth, minHeight() };
+	return { newWidth, top };
 }
 
 void MediaGeneric::draw(Painter &p, const PaintContext &context) const {
@@ -133,7 +117,12 @@ void MediaGeneric::draw(Painter &p, const PaintContext &context) const {
 		const auto radius = st::msgServiceGiftBoxRadius;
 		p.setPen(Qt::NoPen);
 		p.setBrush(context.st->msgServiceBg());
-		p.drawRoundedRect(QRect(0, 0, width(), height()), radius, radius);
+		const auto rect = QRect(0, 0, width(), height());
+		p.drawRoundedRect(rect, radius, radius);
+		//if (context.selected()) {
+		//	p.setBrush(context.st->serviceTextPalette().selectBg);
+		//	p.drawRoundedRect(rect, radius, radius);
+		//}
 	}
 
 	auto translated = 0;
@@ -157,6 +146,11 @@ TextState MediaGeneric::textState(
 		return result;
 	}
 
+	if (_fullAreaLink && QRect(0, 0, width(), height()).contains(point)) {
+		result.link = _fullAreaLink;
+		return result;
+	}
+
 	for (const auto &entry : _entries) {
 		const auto raw = entry.object.get();
 		const auto height = raw->height();
@@ -236,9 +230,11 @@ MediaGenericTextPart::MediaGenericTextPart(
 	QMargins margins,
 	const style::TextStyle &st,
 	const base::flat_map<uint16, ClickHandlerPtr> &links,
-	const Ui::Text::MarkedContext &context)
+	const Ui::Text::MarkedContext &context,
+	style::align align)
 : _text(st::msgMinWidth)
-, _margins(margins) {
+, _margins(margins)
+, _align(align) {
 	_text.setMarkedText(
 		st,
 		text,
@@ -254,12 +250,18 @@ void MediaGenericTextPart::draw(
 		not_null<const MediaGeneric*> owner,
 		const PaintContext &context,
 		int outerWidth) const {
+	const auto use = (width() - _margins.left() - _margins.right());
 	setupPen(p, owner, context);
 	_text.draw(p, {
-		.position = { (outerWidth - width()) / 2, _margins.top() },
+		.position = {
+			((_align == style::al_top)
+				? ((outerWidth - use) / 2)
+				: _margins.left()),
+			_margins.top(),
+		},
 		.outerWidth = outerWidth,
-		.availableWidth = width(),
-		.align = style::al_top,
+		.availableWidth = use,
+		.align = _align,
 		.palette = &(owner->service()
 			? context.st->serviceTextPalette()
 			: context.messageStyle()->textPalette),
@@ -267,6 +269,7 @@ void MediaGenericTextPart::draw(
 		.now = context.now,
 		.pausedEmoji = context.paused || On(PowerSaving::kEmojiChat),
 		.pausedSpoiler = context.paused || On(PowerSaving::kChatSpoiler),
+		.elisionLines = elisionLines(),
 	});
 }
 
@@ -280,34 +283,54 @@ void MediaGenericTextPart::setupPen(
 		: context.messageStyle()->historyTextFg);
 }
 
+int MediaGenericTextPart::elisionLines() const {
+	return 0;
+}
+
 TextState MediaGenericTextPart::textState(
 		QPoint point,
 		StateRequest request,
 		int outerWidth) const {
-	point -= QPoint{ (outerWidth - width()) / 2, _margins.top() };
+	const auto use = (width() - _margins.left() - _margins.right());
+	point -= QPoint{
+		((_align == style::al_top)
+			? ((outerWidth - use) / 2)
+			: _margins.left()),
+		_margins.top(),
+	};
 	auto result = TextState();
 	auto forText = request.forText();
-	forText.align = style::al_top;
-	result.link = _text.getState(point, width(), forText).link;
+	forText.align = _align;
+	result.link = _text.getState(point, use, forText).link;
 	return result;
 }
 
 QSize MediaGenericTextPart::countOptimalSize() {
+	const auto lines = elisionLines();
+	const auto height = lines
+		? std::min(_text.minHeight(), lines * _text.style()->font->height)
+		: _text.minHeight();
 	return {
 		_margins.left() + _text.maxWidth() + _margins.right(),
-		_margins.top() + _text.minHeight() + _margins.bottom(),
+		_margins.top() + height + _margins.bottom(),
 	};
 }
 
 QSize MediaGenericTextPart::countCurrentSize(int newWidth) {
 	auto skip = _margins.left() + _margins.right();
-	const auto size = CountOptimalTextSize(
-		_text,
-		st::msgMinWidth,
-		std::max(st::msgMinWidth, newWidth - skip));
+	const auto size = (_align == style::al_top)
+		? Ui::Text::CountOptimalTextSize(
+			_text,
+			st::msgMinWidth,
+			std::max(st::msgMinWidth, newWidth - skip))
+		: QSize(newWidth - skip, _text.countHeight(newWidth - skip));
+	const auto lines = elisionLines();
+	const auto height = lines
+		? std::min(size.height(), lines * _text.style()->font->height)
+		: size.height();
 	return {
 		size.width() + skip,
-		_margins.top() + size.height() + _margins.bottom(),
+		_margins.top() + height + _margins.bottom(),
 	};
 }
 
@@ -448,13 +471,13 @@ void StickerInBubblePart::ensureCreated(Element *replacing) const {
 		return;
 	} else if (const auto data = _lookup()) {
 		const auto sticker = data.sticker;
-		if (const auto info = sticker->sticker()) {
+		if (sticker->sticker()) {
 			const auto skipPremiumEffect = true;
 			_link = data.link;
 			_skipTop = data.skipTop;
 			_sticker.emplace(_parent, sticker, skipPremiumEffect, replacing);
-			if (data.singleTimePlayback) {
-				_sticker->setPlayingOnce(true);
+			if (data.stopOnLastFrame) {
+				_sticker->setStopOnLastFrame(true);
 			}
 			_sticker->initSize(data.size);
 			_sticker->setCustomCachingTag(data.cacheTag);
