@@ -12,6 +12,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "info/media/info_media_provider.h"
 #include "info/media/info_media_list_section.h"
 #include "info/downloads/info_downloads_provider.h"
+#include "info/saved/info_saved_music_provider.h"
 #include "info/stories/info_stories_provider.h"
 #include "info/info_controller.h"
 #include "layout/layout_mosaic.h"
@@ -29,15 +30,18 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/data_download_manager.h"
 #include "data/data_forum_topic.h"
 #include "data/data_saved_sublist.h"
-#include "history/history_item.h"
-#include "history/history_item_helpers.h"
-#include "history/history.h"
+#include "history/view/media/history_view_save_document_action.h"
 #include "history/view/history_view_cursor_state.h"
 #include "history/view/history_view_service_message.h"
+#include "history/history.h"
+#include "history/history_item.h"
+#include "history/history_item_helpers.h"
 #include "media/stories/media_stories_controller.h" // ...TogglePinnedToast.
 #include "media/stories/media_stories_share.h" // PrepareShareBox.
 #include "window/window_session_controller.h"
 #include "window/window_peer_menu.h"
+#include "ui/widgets/menu/menu_add_action_callback.h"
+#include "ui/widgets/menu/menu_add_action_callback_factory.h"
 #include "ui/widgets/popup_menu.h"
 #include "ui/boxes/confirm_box.h"
 #include "ui/controls/delete_message_context_action.h"
@@ -98,6 +102,8 @@ struct ListWidget::DateBadge {
 		not_null<AbstractController*> controller) {
 	if (controller->isDownloads()) {
 		return std::make_unique<Downloads::Provider>(controller);
+	} else if (controller->musicPeer()) {
+		return std::make_unique<Saved::MusicProvider>(controller);
 	} else if (controller->storiesPeer()) {
 		return std::make_unique<Stories::Provider>(controller);
 	} else if (controller->section().type() == Section::Type::GlobalMedia) {
@@ -196,6 +202,9 @@ void ListWidget::start() {
 		setupStoriesTrackIds();
 		trackSession(&session());
 		restart();
+	} else if (_controller->musicPeer()) {
+		trackSession(&session());
+		restart();
 	} else {
 		trackSession(&session());
 
@@ -1096,17 +1105,6 @@ void ListWidget::showContextMenu(
 			link->property(kDocumentLinkMediaProperty).toULongLong())
 		: nullptr;
 	if (lnkPhoto || lnkDocument) {
-		auto [isVideo, isVoice, isAudio] = [&] {
-			if (lnkDocument) {
-				return std::make_tuple(
-					lnkDocument->isVideoFile(),
-					lnkDocument->isVoiceMessage(),
-					lnkDocument->isAudioFile()
-				);
-			}
-			return std::make_tuple(false, false, false);
-		}();
-
 		if (lnkPhoto) {
 		} else {
 			if (lnkDocument->loading()) {
@@ -1144,16 +1142,11 @@ void ListWidget::showContextMenu(
 							DocumentSaveClickHandler::Mode::ToNewFile);
 					});
 				if (_provider->allowSaveFileAs(item, lnkDocument)) {
-					_contextMenu->addAction(
-						(isVideo
-							? tr::lng_context_save_video(tr::now)
-							: isVoice
-							? tr::lng_context_save_audio(tr::now)
-							: isAudio
-							? tr::lng_context_save_audio_file(tr::now)
-							: tr::lng_context_save_file(tr::now)),
-						std::move(handler),
-						&st::menuIconDownload);
+					HistoryView::AddSaveDocumentAction(
+						Ui::Menu::CreateAddActionCallback(_contextMenu),
+						item,
+						lnkDocument,
+						_controller->parentController());
 				}
 			}
 		}
@@ -1302,6 +1295,10 @@ void ListWidget::showContextMenu(
 		}
 	}
 
+	if (_contextMenu->empty()) {
+		_contextMenu = nullptr;
+		return;
+	}
 	_contextMenu->setDestroyedCallback(crl::guard(
 		this,
 		[=] {
