@@ -7,6 +7,9 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "info/peer_gifts/info_peer_gifts_common.h"
 
+#include "api/api_global_privacy.h"
+#include "api/api_premium.h"
+#include "base/unixtime.h"
 #include "boxes/send_credits_box.h" // SetButtonMarkedLabel
 #include "boxes/star_gift_box.h"
 #include "boxes/sticker_set_box.h"
@@ -18,10 +21,12 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/data_document.h"
 #include "data/data_document_media.h"
 #include "data/data_session.h"
+#include "data/data_user.h"
 #include "history/view/media/history_view_sticker_player.h"
 #include "lang/lang_keys.h"
 #include "info/channel_statistics/earn/earn_icons.h"
 #include "main/main_session.h"
+#include "overview/overview_checkbox.h"
 #include "settings/settings_credits_graphics.h"
 #include "ui/layers/generic_box.h"
 #include "ui/text/format_values.h"
@@ -32,8 +37,10 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "ui/effects/premium_graphics.h"
 #include "ui/painter.h"
 #include "window/window_session_controller.h"
+#include "styles/style_chat.h"
 #include "styles/style_credits.h"
 #include "styles/style_layers.h"
+#include "styles/style_overview.h"
 #include "styles/style_premium.h"
 
 namespace Info::PeerGifts {
@@ -41,6 +48,26 @@ namespace {
 
 constexpr auto kGiftsPerRow = 3;
 
+[[nodiscard]] bool AllowedToSend(
+		const GiftTypeStars &gift,
+		not_null<PeerData*> peer) {
+	using Type = Api::DisallowedGiftType;
+	const auto user = peer->asUser();
+	if (!user || user->isSelf()) {
+		return true;
+	}
+	const auto disallowedTypes = user ? user->disallowedGiftTypes() : Type();
+	const auto allowLimited = !(disallowedTypes & Type::Limited);
+	const auto allowUnlimited = !(disallowedTypes & Type::Unlimited);
+	const auto allowUnique = !(disallowedTypes & Type::Unique);
+	if (gift.resale) {
+		return allowUnique;
+	} else if (!gift.info.limitedCount) {
+		return allowUnlimited;
+	}
+	return allowLimited || (gift.info.starsToUpgrade && allowUnique);
+}
+
 } // namespace
 
 std::strong_ordering operator<=>(const GiftBadge &a, const GiftBadge &b) {
@@ -67,24 +94,99 @@ std::strong_ordering operator<=>(const GiftBadge &a, const GiftBadge &b) {
 	return a.gradient <=> b.gradient;
 }
 
+rpl::producer<std::vector<GiftTypeStars>> GiftsStars(
+		not_null<Main::Session*> session,
+		not_null<PeerData*> peer) {
+	struct Session {
+		std::vector<GiftTypeStars> last;
+	};
+	static auto Map = base::flat_map<not_null<Main::Session*>, Session>();
+
+	const auto filtered = [=](std::vector<GiftTypeStars> list) {
+		list.erase(ranges::remove_if(list, [&](const GiftTypeStars &gift) {
+			return !AllowedToSend(gift, peer);
+		}), end(list));
+		return list;
+	};
+	return [=](auto consumer) {
+		auto lifetime = rpl::lifetime();
+
+		auto i = Map.find(session);
+		if (i == end(Map)) {
+			i = Map.emplace(session, Session()).first;
+			session->lifetime().add([=] { Map.remove(session); });
+		}
+		if (!i->second.last.empty()) {
+			consumer.put_next(filtered(i->second.last));
+		}
+
+		using namespace Api;
+		const auto api = lifetime.make_state<PremiumGiftCodeOptions>(peer);
+		api->requestStarGifts(
+		) | rpl::start_with_error_done([=](QString error) {
+			consumer.put_next({});
+		}, [=] {
+			auto list = std::vector<GiftTypeStars>();
+			const auto &gifts = api->starGifts();
+			list.reserve(gifts.size());
+			for (auto &gift : gifts) {
+				list.push_back({ .info = gift });
+				if (gift.resellCount > 0) {
+					list.push_back({ .info = gift, .resale = true });
+				}
+			}
+			ranges::stable_sort(list, [](const auto &a, const auto &b) {
+				const auto soldOut = [](const auto &gift) {
+					return gift.info.soldOut && !gift.resale;
+				};
+				return soldOut(a) < soldOut(b);
+			});
+
+			auto &map = Map[session];
+			if (map.last != list || list.empty()) {
+				map.last = list;
+				consumer.put_next(filtered(std::move(list)));
+			}
+		}, lifetime);
+
+		return lifetime;
+	};
+}
+
 GiftButton::GiftButton(
 	QWidget *parent,
 	not_null<GiftButtonDelegate*> delegate)
 : AbstractButton(parent)
-, _delegate(delegate) {
+, _delegate(delegate)
+, _lockedTimer([=] { refreshLocked(); }) {
 }
 
 GiftButton::~GiftButton() {
 	unsubscribe();
 }
 
+void GiftButton::onStateChanged(State was, StateChangeSource source) {
+	if (_check) {
+		const auto diff = state() ^ was;
+		if (diff & State::Enum::Over) {
+			_check->setActive(state() & State::Enum::Over);
+		}
+		if (diff & State::Enum::Down) {
+			_check->setPressed(state() & State::Enum::Down);
+		}
+	}
+}
+
 void GiftButton::unsubscribe() {
-	if (base::take(_subscribed)) {
+	if (_subscribed) {
+		_subscribed = false;
 		_userpic->subscribeToUpdates(nullptr);
 	}
 }
 
 void GiftButton::setDescriptor(const GiftDescriptor &descriptor, Mode mode) {
+	_mode = mode;
+
 	const auto unique = v::is<GiftTypeStars>(descriptor)
 		? v::get<GiftTypeStars>(descriptor).info.unique.get()
 		: nullptr;
@@ -106,7 +208,6 @@ void GiftButton::setDescriptor(const GiftDescriptor &descriptor, Mode mode) {
 	_descriptor = descriptor;
 	_resalePrice = resalePrice;
 	const auto resale = (_resalePrice > 0);
-	_small = (mode != Mode::Full);
 	v::match(descriptor, [&](const GiftTypePremium &data) {
 		const auto months = data.months;
 		_text = Ui::Text::String(st::giftBoxGiftHeight / 4);
@@ -143,16 +244,17 @@ void GiftButton::setDescriptor(const GiftDescriptor &descriptor, Mode mode) {
 			{ 0., anim::with_alpha(st::windowActiveTextFg->c, .3) },
 			{ 1., st::windowActiveTextFg->c },
 		});
+		_lockedUntilDate = 0;
 	}, [&](const GiftTypeStars &data) {
 		const auto soldOut = data.info.limitedCount
 			&& !data.userpic
 			&& !data.info.limitedLeft;
-		_userpic = !data.userpic
+		_userpic = (!data.userpic || _mode == GiftButtonMode::Selection)
 			? nullptr
 			: data.from
 			? Ui::MakeUserpicThumbnail(data.from)
 			: Ui::MakeHiddenAuthorThumbnail();
-		if (_small && !resale) {
+		if (small() && !resale) {
 			_price = {};
 			_stars.reset();
 			return;
@@ -169,7 +271,7 @@ void GiftButton::setDescriptor(const GiftDescriptor &descriptor, Mode mode) {
 								? unique->starsForResale
 								: data.info.starsResellMin)
 						).append(data.info.resellCount > 1 ? "+" : ""))
-				: (_small && unique && unique->starsForResale)
+				: (small() && unique && unique->starsForResale)
 				? Data::FormatGiftResaleAsked(*unique)
 				: unique
 				? tr::lng_gift_transfer_button(
@@ -196,8 +298,11 @@ void GiftButton::setDescriptor(const GiftDescriptor &descriptor, Mode mode) {
 			_stars->setColorOverride(
 				Ui::Premium::CreditsIconGradientStops());
 		}
+		_lockedUntilDate = data.info.lockedUntilDate;
 	});
 
+	refreshLocked();
+
 	_resolvedDocument = nullptr;
 	_documentLifetime = _delegate->sticker(
 		descriptor
@@ -214,7 +319,7 @@ void GiftButton::setDescriptor(const GiftDescriptor &descriptor, Mode mode) {
 	_uniquePatternEmoji = nullptr;
 	_uniquePatternCache.clear();
 
-	if (_small && !resale) {
+	if (small() && !resale) {
 		_button = QRect();
 		return;
 	}
@@ -225,7 +330,7 @@ void GiftButton::setDescriptor(const GiftDescriptor &descriptor, Mode mode) {
 		QSize(buttonw, buttonh)
 	).marginsAdded(st::giftBoxButtonPadding);
 	const auto skipy = _delegate->buttonSize().height()
-		- (_small
+		- (small()
 			? st::giftBoxButtonBottomSmall
 			: _byStars.isEmpty()
 			? st::giftBoxButtonBottom
@@ -240,6 +345,22 @@ void GiftButton::setDescriptor(const GiftDescriptor &descriptor, Mode mode) {
 	}
 }
 
+void GiftButton::refreshLocked() {
+	_lockedTimer.cancel();
+
+	const auto lockedFor = _lockedUntilDate
+		? std::max(_lockedUntilDate - base::unixtime::now(), TimeId())
+		: TimeId();
+	const auto locked = (lockedFor > 0);
+	if (locked) {
+		_lockedTimer.callOnce(std::min(lockedFor, 86'400) * crl::time(1000));
+	}
+	if (_locked != locked) {
+		_locked = locked;
+		update();
+	}
+}
+
 void GiftButton::setDocument(not_null<DocumentData*> document) {
 	_resolvedDocument = document;
 	if (_playerDocument == document) {
@@ -295,23 +416,47 @@ void GiftButton::setGeometry(QRect inner, QMargins extend) {
 }
 
 QMargins GiftButton::currentExtend() const {
-	const auto progress = _selectedAnimation.value(_selected ? 1. : 0.);
+	const auto progress = (_selectionMode == GiftSelectionMode::Border)
+		? _selectedAnimation.value(_selected ? 1. : 0.)
+		: 0.;
 	const auto added = anim::interpolate(0, st::giftBoxSelectSkip, progress);
 	return _extend + QMargins(added, added, added, added);
 }
 
-void GiftButton::toggleSelected(bool selected, anim::type animated) {
+bool GiftButton::small() const {
+	return _mode != GiftButtonMode::Full;
+}
+
+void GiftButton::toggleSelected(
+		bool selected,
+		GiftSelectionMode selectionMode,
+		anim::type animated) {
+	_selectionMode = selectionMode;
+	if (_selectionMode != GiftSelectionMode::Check) {
+		_check = nullptr;
+	} else if (!_check) {
+		_check = std::make_unique<Overview::Layout::Checkbox>(
+			[=] { update(); },
+			st::overviewSmallCheck);
+	}
 	if (_selected == selected) {
 		if (animated == anim::type::instant) {
 			_selectedAnimation.stop();
 		}
 		return;
 	}
+
 	const auto duration = st::defaultRoundCheckbox.duration;
 	_selected = selected;
 	if (animated == anim::type::instant) {
+		if (_check) {
+			_check->finishAnimating();
+		}
 		_selectedAnimation.stop();
 		return;
+	} else if (_check) {
+		_check->setChecked(selected, animated);
+		return;
 	}
 	_selectedAnimation.start([=] {
 		update();
@@ -358,7 +503,9 @@ void GiftButton::paintBackground(QPainter &p, const QImage &background) {
 	}
 
 	auto hq = PainterHighQualityEnabler(p);
-	const auto progress = _selectedAnimation.value(_selected ? 1. : 0.);
+	const auto progress = (_selectionMode == GiftSelectionMode::Border)
+		? _selectedAnimation.value(_selected ? 1. : 0.)
+		: 0.;
 	if (progress < 0.01) {
 		return;
 	}
@@ -390,6 +537,48 @@ void GiftButton::contextMenuEvent(QContextMenuEvent *e) {
 		: QCursor::pos());
 }
 
+void GiftButton::mousePressEvent(QMouseEvent *e) {
+	if (_mouseEventsAreListening) {
+		if (e->button() != Qt::LeftButton) {
+			return;
+		}
+		_mouseEvents.fire_copy(e);
+	} else {
+		AbstractButton::mousePressEvent(e);
+	}
+}
+
+void GiftButton::mouseMoveEvent(QMouseEvent *e) {
+	if (_mouseEventsAreListening) {
+		if (e->button() != Qt::LeftButton) {
+			return;
+		}
+		_mouseEvents.fire_copy(e);
+	} else {
+		AbstractButton::mouseMoveEvent(e);
+	}
+}
+
+void GiftButton::mouseReleaseEvent(QMouseEvent *e) {
+	if (_mouseEventsAreListening) {
+		if (e->button() != Qt::LeftButton) {
+			return;
+		}
+		_mouseEvents.fire_copy(e);
+	} else {
+		AbstractButton::mouseReleaseEvent(e);
+	}
+}
+
+rpl::producer<QPoint> GiftButton::contextMenuRequests() const {
+	return _contextMenuRequests.events();
+}
+
+rpl::producer<QMouseEvent*> GiftButton::mouseEvents() {
+	_mouseEventsAreListening = true;
+	return _mouseEvents.events();
+}
+
 void GiftButton::cacheUniqueBackground(
 		not_null<Data::UniqueGift*> unique,
 		int width,
@@ -446,7 +635,7 @@ void GiftButton::paintEvent(QPaintEvent *e) {
 	const auto unique = v::is<GiftTypeStars>(_descriptor)
 		? v::get<GiftTypeStars>(_descriptor).info.unique.get()
 		: nullptr;
-	const auto onsale = (unique && unique->starsForResale && _small);
+	const auto onsale = unique && unique->starsForResale && small();
 	const auto requirePremium = v::is<GiftTypeStars>(_descriptor)
 		&& !v::get<GiftTypeStars>(_descriptor).userpic
 		&& !v::get<GiftTypeStars>(_descriptor).info.unique
@@ -473,6 +662,33 @@ void GiftButton::paintEvent(QPaintEvent *e) {
 		const auto radius = st::giftBoxGiftRadius;
 		p.drawRoundedRect(outer.marginsRemoved(extend), radius, radius);
 	}
+	auto inset = 0;
+	if (_selectionMode == GiftSelectionMode::Inset) {
+		const auto progress = _selectedAnimation.value(_selected ? 1. : 0.);
+		if (progress > 0) {
+			auto hq = PainterHighQualityEnabler(p);
+			auto pen = st::boxBg->p;
+			const auto thickness = style::ConvertScaleExact(2.);
+			pen.setWidthF(progress * thickness);
+			p.setPen(pen);
+			p.setBrush(Qt::NoBrush);
+			const auto height = background.height() / dpr;
+			const auto outer = QRectF(0, 0, width, height);
+			const auto shift = progress * thickness * 2;
+			const auto extend = QMarginsF(currentExtend())
+				+ QMarginsF(shift, shift, shift, shift);
+			const auto radius = st::giftBoxGiftRadius - shift;
+			p.drawRoundedRect(outer.marginsRemoved(extend), radius, radius);
+			inset = int(std::ceil(
+				progress * (thickness * 2 + st::giftBoxUserpicSkip)));
+		}
+	}
+	if (_locked) {
+		st::giftBoxLockIcon.paint(
+			p,
+			position + st::giftBoxLockIconPosition,
+			width);
+	}
 
 	if (_userpic) {
 		if (!_subscribed) {
@@ -482,6 +698,14 @@ void GiftButton::paintEvent(QPaintEvent *e) {
 		const auto image = _userpic->image(st::giftBoxUserpicSize);
 		const auto skip = st::giftBoxUserpicSkip;
 		p.drawImage(extend.left() + skip, extend.top() + skip, image);
+	} else if (_check) {
+		const auto skip = st::giftBoxUserpicSkip;
+		_check->paint(
+			p,
+			QPoint(extend.left() + skip, extend.top() + skip),
+			width,
+			_selected,
+			true);
 	}
 
 	auto frame = QImage();
@@ -502,7 +726,7 @@ void GiftButton::paintEvent(QPaintEvent *e) {
 		p.drawImage(
 			QRect(
 				(width - size.width()) / 2,
-				(_small
+				(small()
 					? st::giftBoxSmallStickerTop
 					: _text.isEmpty()
 					? st::giftBoxStickerStarTop
@@ -516,7 +740,7 @@ void GiftButton::paintEvent(QPaintEvent *e) {
 	if (hidden) {
 		const auto topleft = QPoint(
 			(width - st::giftBoxStickerSize.width()) / 2,
-			(_small
+			(small()
 				? st::giftBoxSmallStickerTop
 				: _text.isEmpty()
 				? st::giftBoxStickerStarTop
@@ -560,6 +784,14 @@ void GiftButton::paintEvent(QPaintEvent *e) {
 				&& !unique
 				&& !data.userpic
 				&& !data.info.limitedLeft;
+			const auto yourLeft = data.info.perUserTotal
+				? (data.info.perUserRemains
+					? tr::lng_gift_stars_your_left(
+						tr::now,
+						lt_count,
+						data.info.perUserRemains)
+					: tr::lng_gift_stars_your_finished(tr::now))
+				: QString();
 			return GiftBadge{
 				.text = (onsale
 					? tr::lng_gift_stars_on_sale(tr::now)
@@ -572,9 +804,13 @@ void GiftButton::paintEvent(QPaintEvent *e) {
 					: (!data.userpic
 						&& !data.info.unique
 						&& data.info.requirePremium)
-					? tr::lng_gift_stars_premium(tr::now)
+					? ((yourLeft.isEmpty() || !_delegate->amPremium())
+						? tr::lng_gift_stars_premium(tr::now)
+						: yourLeft)
 					: (!data.userpic && !data.info.unique)
-					? tr::lng_gift_stars_limited(tr::now)
+					? (yourLeft.isEmpty()
+						? tr::lng_gift_stars_limited(tr::now)
+						: yourLeft)
 					: (count == 1)
 					? tr::lng_gift_limited_of_one(tr::now)
 					: tr::lng_gift_limited_of_count(
@@ -630,7 +866,7 @@ void GiftButton::paintEvent(QPaintEvent *e) {
 	v::match(_descriptor, [](const GiftTypePremium &) {
 	}, [&](const GiftTypeStars &data) {
 		if (!unique) {
-		} else if (data.pinned) {
+		} else if (data.pinned && _mode != GiftButtonMode::Selection) {
 			auto hq = PainterHighQualityEnabler(p);
 			const auto &icon = st::giftBoxPinIcon;
 			const auto skip = st::giftBoxUserpicSkip;
@@ -646,11 +882,11 @@ void GiftButton::paintEvent(QPaintEvent *e) {
 			&& unique->nanoTonForResale
 			&& unique->onlyAcceptTon) {
 			if (_tonIcon.isNull()) {
-				_tonIcon = st::tonIconEmoji.icon.instance(
+				_tonIcon = st::tonIconEmojiLarge.icon.instance(
 					QColor(255, 255, 255));
 			}
 			const auto size = _tonIcon.size() / _tonIcon.devicePixelRatio();
-			const auto skip = st::giftBoxUserpicSkip;
+			const auto skip = st::giftBoxUserpicSkip + inset;
 			const auto add = (st::giftBoxUserpicSize - size.width()) / 2;
 			p.setPen(Qt::NoPen);
 			p.setBrush(unique->backdrop.patternColor);
@@ -742,6 +978,10 @@ Delegate::Delegate(not_null<Main::Session*> session, GiftButtonMode mode)
 	st::giftBoxHiddenMark,
 	RectPart::Center))
 , _mode(mode) {
+	_ministarEmoji = _emojiHelper.paletteDependent(
+		Ui::Earn::IconCreditsEmojiSmall());
+	_starEmoji = _emojiHelper.paletteDependent(
+		Ui::Earn::IconCreditsEmoji());
 }
 
 Delegate::Delegate(Delegate &&other) = default;
@@ -749,7 +989,7 @@ Delegate::Delegate(Delegate &&other) = default;
 Delegate::~Delegate() = default;
 
 TextWithEntities Delegate::star() {
-	return _session->data().customEmojiManager().creditsEmoji();
+	return _starEmoji;
 }
 
 TextWithEntities Delegate::monostar() {
@@ -761,13 +1001,11 @@ TextWithEntities Delegate::monoton() {
 }
 
 TextWithEntities Delegate::ministar() {
-	const auto owner = &_session->data();
-	const auto top = st::giftBoxByStarsStarTop;
-	return owner->customEmojiManager().ministarEmoji({ 0, top, 0, 0 });
+	return _ministarEmoji;
 }
 
 Ui::Text::MarkedContext Delegate::textContext() {
-	return Core::TextContext({ .session = _session });
+	return _emojiHelper.context();
 }
 
 QSize Delegate::buttonSize() {
@@ -779,14 +1017,14 @@ QSize Delegate::buttonSize() {
 	const auto available = width - padding.left() - padding.right();
 	const auto singlew = (available - 2 * st::giftBoxGiftSkip.x())
 		/ kGiftsPerRow;
-	const auto minimal = (_mode == GiftButtonMode::Minimal);
+	const auto minimal = (_mode != GiftButtonMode::Full);
 	_single = QSize(
 		singlew,
 		minimal ? st::giftBoxGiftSmall : st::giftBoxGiftHeight);
 	return _single;
 }
 
-QMargins Delegate::buttonExtend() {
+QMargins Delegate::buttonExtend() const {
 	return st::defaultDropdownMenu.wrap.shadow.extend;
 }
 
@@ -862,11 +1100,17 @@ QImage Delegate::cachedBadge(const GiftBadge &badge) {
 	auto &image = _badges[badge];
 	if (image.isNull()) {
 		const auto &extend = buttonExtend();
-		image = ValidateRotatedBadge(badge, extend.top());
+		const auto line = st::lineWidth;
+		const auto padding = QMargins(extend.top(), 0, extend.top(), line);
+		image = ValidateRotatedBadge(badge, padding);
 	}
 	return image;
 }
 
+bool Delegate::amPremium() {
+	return _session->premium();
+}
+
 DocumentData *LookupGiftSticker(
 		not_null<Main::Session*> session,
 		const GiftDescriptor &descriptor) {
@@ -904,17 +1148,19 @@ rpl::producer<not_null<DocumentData*>> GiftStickerValue(
 	});
 }
 
-QImage ValidateRotatedBadge(const GiftBadge &badge, int added) {
+QImage ValidateRotatedBadge(const GiftBadge &badge, QMargins padding) {
 	const auto &font = badge.small
 		? st::giftBoxGiftBadgeFont
-		: st::semiboldFont;
-	const auto twidth = font->width(badge.text) + 2 * added;
+		: st::msgServiceGiftBoxBadgeFont;
+	const auto twidth = font->width(badge.text)
+		+ padding.left()
+		+ padding.right();
 	const auto skip = int(std::ceil(twidth / M_SQRT2));
 	const auto ratio = style::DevicePixelRatio();
 	const auto multiplier = ratio * 3;
 	const auto size = (twidth + font->height * 2);
-	const auto height = font->height + st::lineWidth;
-	const auto textpos = QPoint(size - skip, added);
+	const auto height = padding.top() + font->height + padding.bottom();
+	const auto textpos = QPoint(size - skip, padding.top());
 	auto image = QImage(
 		QSize(size, size) * multiplier,
 		QImage::Format_ARGB32_Premultiplied);
@@ -927,7 +1173,9 @@ QImage ValidateRotatedBadge(const GiftBadge &badge, int added) {
 		p.rotate(45.);
 		p.setFont(font);
 		p.setPen(badge.fg);
-		p.drawText(QPoint(added, font->ascent), badge.text);
+		p.drawText(
+			QPoint(padding.left(), padding.top() + font->ascent),
+			badge.text);
 	}
 
 	auto scaled = image.scaled(
@@ -1007,13 +1255,15 @@ void SelectGiftToUnpin(
 				box,
 				tr::lng_gift_many_pinned_title(),
 				st::giftBoxSubtitle),
-			st::giftBoxSubtitleMargin);
+			st::giftBoxSubtitleMargin,
+			style::al_top);
 		box->addRow(
 			object_ptr<Ui::FlatLabel>(
 				box,
 				tr::lng_gift_many_pinned_choose(),
 				st::giftTooManyPinnedChoose),
-			st::giftBoxAboutMargin);
+			st::giftBoxAboutMargin,
+			style::al_top);
 
 		const auto gifts = box->addRow(
 			object_ptr<Ui::RpWidget>(box),
