@@ -45,6 +45,7 @@ struct InfoProfileCover;
 namespace Info::Profile {
 
 class EmojiStatusPanel;
+class MusicButton;
 class Badge;
 
 class TopicIconView final {
@@ -140,6 +141,7 @@ private:
 
 	void setupShowLastSeen();
 	void setupChildGeometry();
+	void setupSavedMusic();
 	void initViewers(rpl::producer<QString> title);
 	void refreshStatusText();
 	void refreshNameGeometry(int newWidth);
@@ -179,6 +181,8 @@ private:
 	//object_ptr<CoverDropArea> _dropArea = { nullptr };
 	base::Timer _refreshStatusTimer;
 
+	std::unique_ptr<MusicButton> _musicButton;
+
 	rpl::event_stream<Section> _showSection;
 
 };
