@@ -33,7 +33,7 @@ SessionSettings::SessionSettings()
 
 QByteArray SessionSettings::serialize() const {
 	const auto autoDownload = _autoDownload.serialize();
-	const auto size = sizeof(qint32) * 4
+	auto size = sizeof(qint32) * 4
 		+ _groupStickersSectionHidden.size() * sizeof(quint64)
 		+ sizeof(qint32) * 4
 		+ Serialize::bytearraySize(autoDownload)
@@ -44,7 +44,25 @@ QByteArray SessionSettings::serialize() const {
 		+ sizeof(qint32) * 3
 		+ _hiddenPinnedMessages.size() * (sizeof(quint64) * 4)
 		+ sizeof(qint32)
-		+ _verticalSubsectionTabs.size() * sizeof(quint64);
+		+ _verticalSubsectionTabs.size() * sizeof(quint64)
+		+ sizeof(qint32) // _ringtoneDefaultVolumes size
+		+ (_ringtoneDefaultVolumes.size()
+			* (0
+				+ sizeof(uint8_t) * 1 // Data::DefaultNotify
+				+ sizeof(ushort))) // Volume
+		+ sizeof(qint32) // _ringtoneVolumes size
+		+ (_ringtoneVolumes.size()
+			* (0
+				+ sizeof(quint64) * 3 // ThreadId
+				+ sizeof(ushort))) // Volume
+		+ sizeof(qint32) // _ratedTranscriptions size
+		+ (_ratedTranscriptions.size() * sizeof(quint64))
+		+ sizeof(qint32); // _unreviewed size
+	for (const auto &auth : _unreviewed) {
+		size += sizeof(quint64) + sizeof(qint32) + sizeof(qint32)
+			+ Serialize::stringSize(auth.device)
+			+ Serialize::stringSize(auth.location);
+	}
 
 	auto result = QByteArray();
 	result.reserve(size);
@@ -100,6 +118,31 @@ QByteArray SessionSettings::serialize() const {
 		for (const auto &peerId : _verticalSubsectionTabs) {
 			stream << SerializePeerId(peerId);
 		}
+		stream << qint32(_ringtoneDefaultVolumes.size());
+		for (const auto &[key, value] : _ringtoneDefaultVolumes) {
+			stream << uint8_t(key) << ushort(value);
+		}
+		stream << qint32(_ringtoneVolumes.size());
+		for (const auto &[key, value] : _ringtoneVolumes) {
+			stream
+				<< SerializePeerId(key.peerId)
+				<< qint64(key.topicRootId.bare)
+				<< SerializePeerId(key.monoforumPeerId)
+				<< ushort(value);
+		}
+		stream << qint32(_ratedTranscriptions.size());
+		for (const auto &transcriptionId : _ratedTranscriptions) {
+			stream << quint64(transcriptionId);
+		}
+		stream << qint32(_unreviewed.size());
+		for (const auto &auth : _unreviewed) {
+			stream
+				<< quint64(auth.hash)
+				<< qint32(auth.unconfirmed ? 1 : 0)
+				<< qint32(auth.date)
+				<< auth.device
+				<< auth.location;
+		}
 	}
 
 	Ensures(result.size() == size);
@@ -167,6 +210,10 @@ void SessionSettings::addFromSerialized(const QByteArray &serialized) {
 	qint32 legacySkipPremiumStickersSet = 0;
 	qint32 lastNonPremiumLimitDownload = 0;
 	qint32 lastNonPremiumLimitUpload = 0;
+	base::flat_map<Data::DefaultNotify, ushort> ringtoneDefaultVolumes;
+	base::flat_map<ThreadId, ushort> ringtoneVolumes;
+	base::flat_set<uint64> ratedTranscriptions;
+	std::vector<Data::UnreviewedAuth> unreviewed;
 
 	stream >> versionTag;
 	if (versionTag == kVersionTag) {
@@ -395,7 +442,7 @@ void SessionSettings::addFromSerialized(const QByteArray &serialized) {
 		stream >> count;
 		if (stream.status() == QDataStream::Ok) {
 			for (auto i = 0; i != count; ++i) {
-				quint64 period;
+				auto period = quint64();
 				stream >> period;
 				mutePeriods.emplace_back(period);
 			}
@@ -489,6 +536,97 @@ void SessionSettings::addFromSerialized(const QByteArray &serialized) {
 			}
 		}
 	}
+	if (!stream.atEnd()) {
+		auto count = qint32(0);
+		stream >> count;
+		if (stream.status() == QDataStream::Ok) {
+			for (auto i = 0; i != count; ++i) {
+				auto keyDefaultNotify = uint8_t();
+				auto value = ushort();
+				stream
+					>> keyDefaultNotify
+					>> value;
+				if (stream.status() != QDataStream::Ok) {
+					LOG(("App Error: "
+						"Bad data for SessionSettings::addFromSerialized()"));
+					return;
+				}
+				ringtoneDefaultVolumes.emplace(
+					static_cast<Data::DefaultNotify>(keyDefaultNotify),
+					value);
+			}
+		}
+	}
+	if (!stream.atEnd()) {
+		auto count = qint32(0);
+		stream >> count;
+		if (stream.status() == QDataStream::Ok) {
+			for (auto i = 0; i != count; ++i) {
+				auto keyPeerId = quint64();
+				auto keyTopicRootId = qint64();
+				auto keyMonoforumPeerId = quint64();
+				auto value = ushort();
+				stream
+					>> keyPeerId
+					>> keyTopicRootId
+					>> keyMonoforumPeerId
+					>> value;
+				if (stream.status() != QDataStream::Ok) {
+					LOG(("App Error: "
+						"Bad data for SessionSettings::addFromSerialized()"));
+					return;
+				}
+				ringtoneVolumes.emplace(ThreadId{
+					DeserializePeerId(keyPeerId),
+					keyTopicRootId,
+					DeserializePeerId(keyMonoforumPeerId),
+				}, value);
+			}
+		}
+	}
+	if (!stream.atEnd()) {
+		auto count = qint32(0);
+		stream >> count;
+		if (stream.status() == QDataStream::Ok) {
+			for (auto i = 0; i != count; ++i) {
+				auto transcriptionId = quint64();
+				stream >> transcriptionId;
+				if (stream.status() != QDataStream::Ok) {
+					LOG(("App Error: "
+						"Bad data for SessionSettings::addFromSerialized()"
+						"with ratedTranscriptions"));
+					return;
+				}
+				ratedTranscriptions.emplace(transcriptionId);
+			}
+		}
+	}
+	if (!stream.atEnd()) {
+		auto count = qint32(0);
+		stream >> count;
+		if (stream.status() == QDataStream::Ok) {
+			for (auto i = 0; i != count; ++i) {
+				auto hash = quint64();
+				auto unconfirmed = qint32();
+				auto date = qint32();
+				QString device, location;
+				stream >> hash >> unconfirmed >> date >> device >> location;
+				if (stream.status() != QDataStream::Ok) {
+					LOG(("App Error: "
+						"Bad data for SessionSettings::addFromSerialized()"
+						"with unreviewed"));
+					return;
+				}
+				unreviewed.emplace_back(Data::UnreviewedAuth{
+					.hash = hash,
+					.unconfirmed = (unconfirmed == 1),
+					.date = TimeId(date),
+					.device = device,
+					.location = location,
+				});
+			}
+		}
+	}
 	if (stream.status() != QDataStream::Ok) {
 		LOG(("App Error: "
 			"Bad data for SessionSettings::addFromSerialized()"));
@@ -536,6 +674,10 @@ void SessionSettings::addFromSerialized(const QByteArray &serialized) {
 	_lastNonPremiumLimitDownload = lastNonPremiumLimitDownload;
 	_lastNonPremiumLimitUpload = lastNonPremiumLimitUpload;
 	_verticalSubsectionTabs = std::move(verticalSubsectionTabs);
+	_ringtoneDefaultVolumes = std::move(ringtoneDefaultVolumes);
+	_ringtoneVolumes = std::move(ringtoneVolumes);
+	_ratedTranscriptions = std::move(ratedTranscriptions);
+	_unreviewed = std::move(unreviewed);
 
 	if (version < 2) {
 		app.setLastSeenWarningSeen(appLastSeenWarningSeen == 1);
@@ -710,4 +852,58 @@ void SessionSettings::addMutePeriod(TimeId period) {
 	}
 }
 
+ushort SessionSettings::ringtoneVolume(
+		Data::DefaultNotify defaultNotify) const {
+	const auto i = _ringtoneDefaultVolumes.find(defaultNotify);
+	return (i != end(_ringtoneDefaultVolumes)) ? i->second : 0;
+}
+
+void SessionSettings::setRingtoneVolume(
+		Data::DefaultNotify defaultNotify,
+		ushort volume) {
+	if (volume > 0 && volume <= 100) {
+		_ringtoneDefaultVolumes[defaultNotify] = volume;
+	} else {
+		_ringtoneDefaultVolumes.remove(defaultNotify);
+	}
+}
+
+ushort SessionSettings::ringtoneVolume(
+		PeerId peerId,
+		MsgId topicRootId,
+		PeerId monoforumPeerId) const {
+	const auto i = _ringtoneVolumes.find(
+		ThreadId{ peerId, topicRootId, monoforumPeerId });
+	return (i != end(_ringtoneVolumes)) ? i->second : 0;
+}
+
+void SessionSettings::setRingtoneVolume(
+		PeerId peerId,
+		MsgId topicRootId,
+		PeerId monoforumPeerId,
+		ushort volume) {
+	const auto id = ThreadId{ peerId, topicRootId, monoforumPeerId };
+	if (volume > 0 && volume <= 100) {
+		_ringtoneVolumes[id] = volume;
+	} else {
+		_ringtoneVolumes.remove(id);
+	}
+}
+
+void SessionSettings::markTranscriptionAsRated(uint64 transcriptionId) {
+	_ratedTranscriptions.emplace(transcriptionId);
+}
+
+bool SessionSettings::isTranscriptionRated(uint64 transcriptionId) const {
+	return _ratedTranscriptions.contains(transcriptionId);
+}
+
+void SessionSettings::setUnreviewed(std::vector<Data::UnreviewedAuth> auths) {
+	_unreviewed = std::move(auths);
+}
+
+const std::vector<Data::UnreviewedAuth> &SessionSettings::unreviewed() const {
+	return _unreviewed;
+}
+
 } // namespace Main
