@@ -259,7 +259,7 @@ void MainWindow::setupMain(
 	const auto animated = _intro
 		|| (_passcodeLock && !Core::App().passcodeLocked());
 	const auto weakAnimatedLayer = (_main && _layer && !_passcodeLock)
-		? Ui::MakeWeak(_layer.get())
+		? base::make_weak(_layer.get())
 		: nullptr;
 	if (weakAnimatedLayer) {
 		Assert(!animated);
@@ -288,7 +288,7 @@ void MainWindow::setupMain(
 		Core::App().checkStartUrl();
 	}
 	fixOrder();
-	if (const auto strong = weakAnimatedLayer.data()) {
+	if (const auto strong = weakAnimatedLayer.get()) {
 		strong->hideAllAnimatedRun();
 	}
 }
