@@ -126,6 +126,28 @@ struct MessageImageStyle {
 	style::icon historyPageEnlarge = { Qt::Uninitialized };
 };
 
+struct ColorCollectible {
+	uint64 collectibleId = 0;
+	uint64 giftEmojiId = 0;
+	uint64 backgroundEmojiId = 0;
+	QColor accentColor;
+	std::vector<QColor> strip;
+	QColor darkAccentColor;
+	std::vector<QColor> darkStrip;
+
+	friend inline bool operator==(
+		const ColorCollectible &,
+		const ColorCollectible &) = default;
+};
+
+struct ColorCollectiblePtrCompare {
+	bool operator()(
+			const std::weak_ptr<ColorCollectible> &a,
+			const std::weak_ptr<ColorCollectible> &b) const {
+		return a.owner_before(b);
+	}
+};
+
 struct ReactionPaintInfo {
 	QPoint position;
 	QPoint effectOffset;
@@ -140,7 +162,13 @@ struct BackgroundEmojiCache {
 struct BackgroundEmojiData {
 	std::unique_ptr<Text::CustomEmoji> emoji;
 	QImage firstFrameMask;
-	std::array<BackgroundEmojiCache, 2 * (3 + kColorIndexCount)> caches;
+	base::flat_map<int, BackgroundEmojiCache> caches;
+	base::flat_map<
+		std::weak_ptr<ColorCollectible>,
+		BackgroundEmojiCache,
+		ColorCollectiblePtrCompare> collectibleCaches;
+	std::unique_ptr<Text::CustomEmoji> gift;
+	QImage firstGiftFrame;
 
 	[[nodiscard]] static int CacheIndex(
 		bool selected,
@@ -184,6 +212,7 @@ struct ChatPaintContext {
 	[[nodiscard]] not_null<const MessageStyle*> messageStyle() const;
 	[[nodiscard]] not_null<const MessageImageStyle*> imageStyle() const;
 	[[nodiscard]] not_null<Text::QuotePaintCache*> quoteCache(
+		const std::shared_ptr<ColorCollectible> &colorCollectible,
 		uint8 colorIndex) const;
 
 	[[nodiscard]] ChatPaintContext translated(int x, int y) const {
@@ -214,11 +243,12 @@ struct ChatPaintContext {
 	};
 
 
-	// This is supported only in unwrapped media for now.
+	// This is supported only in unwrapped media and text messages for now.
 	enum class SkipDrawingParts {
 		None,
 		Content,
 		Surrounding,
+		Bubble,
 	};
 	SkipDrawingParts skipDrawingParts = SkipDrawingParts::None;
 
@@ -314,6 +344,8 @@ public:
 	[[nodiscard]] const MessageImageStyle &imageStyle(bool selected) const;
 
 	[[nodiscard]] int colorPatternIndex(uint8 colorIndex) const;
+	[[nodiscard]] int collectiblePatternIndex(
+		const std::shared_ptr<ColorCollectible> &collectible) const;
 	[[nodiscard]] ColorIndexValues computeColorIndexValues(
 		bool selected,
 		uint8 colorIndex) const;
@@ -325,19 +357,31 @@ public:
 	[[nodiscard]] const ColorIndexValues &coloredValues(
 		bool selected,
 		uint8 colorIndex) const;
+	[[nodiscard]] QColor collectibleNameColor(
+		const std::shared_ptr<ColorCollectible> &collectible) const;
 	[[nodiscard]] not_null<Text::QuotePaintCache*> coloredQuoteCache(
 		bool selected,
 		uint8 colorIndex) const;
 	[[nodiscard]] not_null<Text::QuotePaintCache*> coloredReplyCache(
 		bool selected,
 		uint8 colorIndex) const;
+	[[nodiscard]] not_null<Text::QuotePaintCache*> collectibleQuoteCache(
+		bool selected,
+		const std::shared_ptr<ColorCollectible> &collectible) const;
+	[[nodiscard]] not_null<Text::QuotePaintCache*> collectibleReplyCache(
+		bool selected,
+		const std::shared_ptr<ColorCollectible> &collectible) const;
 
 	[[nodiscard]] const style::TextPalette &coloredTextPalette(
 		bool selected,
 		uint8 colorIndex) const;
+	[[nodiscard]] const style::TextPalette &collectibleTextPalette(
+		bool selected,
+		const std::shared_ptr<ColorCollectible> &collectible) const;
 
 	[[nodiscard]] not_null<BackgroundEmojiData*> backgroundEmojiData(
-		uint64 id) const;
+		uint64 emojiId,
+		const std::shared_ptr<ColorCollectible> &collectible) const;
 
 	[[nodiscard]] const CornersPixmaps &msgBotKbOverBgAddCornersSmall() const;
 	[[nodiscard]] const CornersPixmaps &msgBotKbOverBgAddCornersLarge() const;
@@ -444,10 +488,23 @@ private:
 		kColorIndexCount * 2>;
 
 	struct ColoredPalette {
+		ColoredPalette();
+		ColoredPalette(const ColoredPalette &other);
+		ColoredPalette &operator=(const ColoredPalette &other);
+
 		std::optional<style::owned_color> linkFg;
 		style::TextPalette data;
 	};
 
+	struct CollectibleColors {
+		std::unique_ptr<Text::QuotePaintCache> quote;
+		std::unique_ptr<Text::QuotePaintCache> quoteSelected;
+		std::unique_ptr<Text::QuotePaintCache> reply;
+		std::unique_ptr<Text::QuotePaintCache> replySelected;
+		ColoredPalette palette;
+		ColoredPalette paletteSelected;
+	};
+
 	void assignPalette(not_null<const style::palette*> palette);
 	void clearColorIndexCaches();
 	void updateDarkValue();
@@ -456,6 +513,11 @@ private:
 		ColoredQuotePaintCaches &caches,
 		bool selected,
 		uint8 colorIndex) const;
+	[[nodiscard]] not_null<Text::QuotePaintCache*> collectibleCache(
+		std::unique_ptr<Text::QuotePaintCache> &cache,
+		const std::shared_ptr<ColorCollectible> &collectible) const;
+	[[nodiscard]] CollectibleColors &resolveCollectibleCaches(
+		const std::shared_ptr<ColorCollectible> &collectible) const;
 
 	void make(style::color &my, const style::color &original) const;
 	void make(style::icon &my, const style::icon &original) const;
@@ -521,6 +583,10 @@ private:
 	mutable std::array<
 		ColoredPalette,
 		2 * kColorIndexCount> _coloredTextPalettes;
+	mutable base::flat_map<
+		std::weak_ptr<ColorCollectible>,
+		CollectibleColors,
+		ColorCollectiblePtrCompare> _collectibleCaches;
 	mutable base::flat_map<uint64, BackgroundEmojiData> _backgroundEmojis;
 
 	style::TextPalette _historyPsaForwardPalette;
@@ -572,12 +638,20 @@ private:
 [[nodiscard]] QColor FromNameFg(
 	not_null<const ChatStyle*> st,
 	bool selected,
-	uint8 colorIndex);
+	uint8 colorIndex,
+	const std::shared_ptr<Ui::ColorCollectible> &colorCollectible);
 
 [[nodiscard]] inline QColor FromNameFg(
 		const ChatPaintContext &context,
-		uint8 colorIndex) {
-	return FromNameFg(context.st, context.selected(), colorIndex);
+		uint8 colorIndex,
+		const std::shared_ptr<Ui::ColorCollectible> &colorCollectible) {
+	return context.outbg
+		? context.messageStyle()->msgServiceFg->c
+		: FromNameFg(
+			context.st,
+			context.selected(),
+			colorIndex,
+			colorCollectible);
 }
 
 void FillComplexOverlayRect(
